use Test::More tests => 35;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Installer::API::CVSTagHandler;
use DE_EPAGES::Core::API::File qw (GetFileContent DeleteFile WriteFile DeleteDirectory);
use strict;

use constant VCS_TEST_NUMBER => 30;

sub Main {

  SKIP: {
      skip( 'Forced skip of internal VCS tests.', VCS_TEST_NUMBER ) if $ENV{'SKIP_VCS_TESTS'};

      my $TagHandler = DE_EPAGES::Installer::API::CVSTagHandler->new();

      Cleanup($TagHandler);

      my $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      ok( scalar @$aChangedFiles == 0, 'no result if tag doesnt exist.' );

      #Test if files changed, set patch tag to 1.3.2.1
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [ 'patch_test_CVSHandler_new', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      ok($TagHandler->existsTagOnFile('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt','patch_test_CVSHandler_new'),'tag exists on file');
      ok(not ($TagHandler->existsTagOnFile('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt','patch_test_CVSHandler_end')),'tag not exists on file');

      #get results
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );
      my $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new',
        { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if changed file is find
      ok( ( ( scalar @$aChangedFiles == 1 ) and ( $aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ) ),
        'result found in changed files.' );

      #check if no file with unchanged content is find
      ok( scalar @$aChangedFilesUnchangedContent == 0, 'no result with unchanged content exist.' );

      #Test for unchanged content set patch tag to 1.3.2.2
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.2', '-F' => '' },
        [ 'patch_test_CVSHandler_new', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      #get results
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );
      $aChangedFilesUnchangedContent = $TagHandler->getChangedFilesWithUnchangedContent( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new',
        { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if changed file is find
      ok( ( ( scalar @$aChangedFiles == 1 ) and ( $aChangedFiles->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ) ),
        'result found in changed files.' );

      #check if changed file with unchanged content is find
      ok((
            ( scalar @$aChangedFilesUnchangedContent == 1 ) and
            ( $aChangedFilesUnchangedContent->[0] eq 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' )
         ),
         'result found in changed files with unchanged content.'
        );

      #removing patch tag
      $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_new' );

      #check if result is empty
      $aChangedFiles
        = $TagHandler->getChangedFiles( 'release_test_CVSHandler_old', 'patch_test_CVSHandler_new', { 'cvspath' => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t' } );

      #check if result is empty so this is a indirect proof that the patch tag is gone
      ok( scalar @$aChangedFiles == 0, 'patch tag is removed' );

      #check for patch affected file names
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch'),                                'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('\Cartridges\DE_EPAGES\Calendar\Data\Private\BO\sprite_Calendar.css'),                                   'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\ProductPortal\API\Cartridge.pm'),                         'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\Ebay\Database\XML\ActionsEbayAuction.xml'),               'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\FactFinder\Database\mysql\Tables\factfinderchannel.sql'), 'patch affected file.' );
      ok( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\IGFS\Patches\Patch_6_12_3\patch.xml'),                    'patch affected file.' );
      ok( not( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS\Cartridges\DE_EPAGES\CreditCheck\Templates\MBO\MBO-CreditCheckAgencyConfigCustomerGroups.TabPage.en.xml') ),
        'no patch affected file.' );
      ok( not( $TagHandler->patchAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml') ),
        'no patch affected file.' );

      ok( not( $TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Patch/src/Shared/Config/Scheduler.conf-patch') ), 'no inspection affected file.' );
      ok( $TagHandler->inspectionAffectedFile('RND/epages/epages5/AllOS/Languages/ca/Cartridges/DE_EPAGES/Presentation/Database/XML/Translation.HelptopicsShop.ca.xml'),
        'inspection affected file.' );

      #get hot fix tags
      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' }, [ 'release_testtag_88_88_88', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      my $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
      };

      is_deeply( $TagHandler->getInfoForTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' ),
        $hInfo, 'correct engineer and jira task' );

      $TagHandler->command(
        'rtag',
        { '-r' => '1.3.2.1', '-F' => '' },
        [ 'patch_test_CVSHandler_end', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      $hInfo = {
        'engineer' => 'cbordis',
        'jiratask' => 'EPG-19048'
      };

      is_deeply( $TagHandler->getInfoForTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end' ),
        $hInfo, 'correct engineer and jira task' );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' }, [ 'release_testtag_99_99_99', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_1', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_1_1', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_2', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command( 'rtag', { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_3', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ] );

      #set fictional release tag to start revision
      $TagHandler->command(
        'rtag',
        { '-r' => '1.3', '-F' => '' },
        [ 'release_testtag_99_99_99_10', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
      );

      ok((
        $TagHandler->getLatestCorrespondingHotFixTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' ) eq
        'release_testtag_88_88_88'
        ),
        'return release tag if no hot fix tag exists.'
      );
      ok((
        $TagHandler->getLatestCorrespondingHotFixTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99' ) eq
        'release_testtag_99_99_99_10'
        ),
        'correct result for release tag'
      );

      #revision test
      ok(( $TagHandler->getRevision( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10' ) eq '1.3' ),
        'correct revision' );

      ok( $TagHandler->isDirectSuccessor( '1.13.8.3.2.4', '1.13.8.3.2.5' ), 'is direct successor correct' );

      ok( $TagHandler->isDirectSuccessor( '1.13.8.3.2.4', '1.13.8.3.2.3' ) eq '', 'is direct successor correct' );

      ok( $TagHandler->getPreviousRevision('1.13.8.3.2.1') eq '1.13.8.3', 'get previous revision correct' );

      my $aInput = [ '1.2.4.1', '1.2.4.2', '1.2.4.3', '1.2.4.4', '1.2.4.8', '1.2.4.9', '1.2.4.14' ];
      my $aOutput = { '1.2' => '1.2.4.4', '1.2.4.7' => '1.2.4.9', '1.2.4.13' => '1.2.4.14' };
      is_deeply( $TagHandler->getMergeRevisions($aInput), $aOutput, 'get merge revisions correct' );

      $aInput = [ '1.2.1', '1.2.2', '1.2.3' ];
      $aOutput = { '1' => '1.2.3' };
      is_deeply( $TagHandler->getMergeRevisions($aInput), $aOutput, 'get merge revisions correct' );

      ok( $TagHandler->getBranchForHotfix('hotfix_6_14_3_4') eq 'B_patch_6_14_3', 'get branch for hotfix correct' );
      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/Permissions2_new.xml', 'HEAD');
      my $hDiffDelete = {
        'deleted' => { '136' => '<     <RoleAction Class="Appointment" Action="ViewPrint"          delete="1" />' }
      };
      is_deeply( $TagHandler->getRevisionDiffAsHash( '1.1.4.1', '1.3', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/Permissions2_new.xml' ),$hDiffDelete,'get revision diff as hash correct for deleted parts');
      
      my $hDiffAdd = {
        'added' => { '136' => '>     <RoleAction Class="Appointment" Action="ViewPrint"          delete="1" />' }
      };
      is_deeply( $TagHandler->getRevisionDiffAsHash( '1.3', '1.1.4.1', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/Permissions2_new.xml' ),$hDiffAdd,'get revision diff as hash correct for added parts');
      
      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/Dictionary_target.en.xml', 'HEAD');
      my $hDiffChange = {
        'changed' => { 
        'old' => { '7' => '<     <Translation Keyword="ToProduct">Goto product</Translation>' },
        'new' => { '9' => '>     <Translation Keyword="ToProduct">Go to product</Translation>' }
        }
      };
      is_deeply( $TagHandler->getRevisionDiffAsHash( '1.2', '1.3', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/Dictionary_target.en.xml' ),$hDiffChange,'get revision diff as hash correct for changed parts');

      $TagHandler->checkout( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'EPG-23291', '/tmp/Test/' );
      my $rString1    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change1.txt' );
      my $rString2    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change2.txt' );
      my $rString3    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change3.txt' );
      my $rString4    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change4.txt' );
      my $rString5    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change5.txt' );
      my $rString6    = GetFileContent( $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/test_CVSHandler_Change6.txt' );

      foreach my $String ( @{ [ $rString1, $rString2, $rString3, $rString4, $rString5, $rString6, ] } ) {
        WriteFile( '/tmp/Test/test_CVSHandler.txt', $String );
        $TagHandler->commit( 'EPG-23291: test case', '/tmp/Test/test_CVSHandler.txt' );
      }
      DeleteFile('/tmp/Test/test_CVSHandler.txt');

      my $Comment = $TagHandler->getCommentForRevision('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', '1.3');

      ok('EPG-19048 Test file for CVS testing. base file' eq $Comment,'correct comment');

      $Comment = $TagHandler->getCommentForRevision('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', '1.3.64.2');

      ok('Merged HEAD' eq $Comment,'correct comment 2nd');


      ok(
          (
              $TagHandler->getLatestReleaseTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt') eq
              $TagHandler->getLatestReleaseTag('RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/API/Cartridge.pm')
          ),
          'correct latest release tags'
      );

      Cleanup($TagHandler);
  }
}

sub Cleanup {
  my ($TagHandler) = @_;

  #remove fictional patch tag
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t',                     'patch_test_CVSHandler_new' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t',                     'release_test_CVSHandler_old' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_88_88_88' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_1_1' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_2' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_3' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'release_testtag_99_99_99_10' );
  $TagHandler->removeTag( 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt', 'patch_test_CVSHandler_end' );

  #set fictional release tag to start revision
  $TagHandler->command(
    'rtag',
    { '-r' => '1.1', '-F' => '' },
    [ 'release_test_CVSHandler_old', 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/test_CVSHandler.txt' ]
  );

  $TagHandler->deleteRevisions( '1.3.46.2', '1.3.46.19', '/tmp/Test/test_CVSHandler.txt' );
  $TagHandler->deleteRevisions( '1.3.48.2', '1.3.48.2', '/tmp/Test/test_CVSHandler.txt' );
  DeleteTempCheckoutFiles('/tmp/Test/');

}

sub DeleteTempCheckoutFiles {
  my ($CheckoutFileDirectory) = @_;

  #add slash if necessary
  $CheckoutFileDirectory .= '\\' unless ( $CheckoutFileDirectory =~ /(\\|\/)$/ );
  DeleteFile( $CheckoutFileDirectory . '.#test_CVSHandler.txt.1.3.48.1' );
  DeleteFile( $CheckoutFileDirectory . 'test_CVSHandler.txt' );
  DeleteDirectory( $CheckoutFileDirectory . 'CVS', { 'recursive' => 1 } );
}

RunScript( Sub => \&Main );

done_testing();

1;
