#========================================================================================
# package      DE_EPAGES::Ipayment::t::unit::Database_BasketOrOrder;
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Database::BasketOrOrder
#========================================================================================
package DE_EPAGES::Ipayment::t::unit::Database_BasketOrOrder;
use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;

sub Main {
    test_Extraction();
    test_ExtractionWithInvalidBasketOrOrder();
    test_ExtractionWithNoBasket();

    done_testing();
}


sub test_Extraction {
    my $SubTestName = (caller(0))[3];

    my $MockedBasket = _GetMockedBasket();
    my $DatabaseBasketOrOrder = _GetDatabaseBasketOrOrder();

    my $ExtractionWasSuccessful = $DatabaseBasketOrOrder->extractFromBasketOrOrder($MockedBasket);

    ## no critic (ProhibitLongLines)
    ok($ExtractionWasSuccessful, "$SubTestName - was it possible to extract data from the database object - yes");
    is($DatabaseBasketOrOrder->getAccountID(), 99999, "$SubTestName - AccountID has expected value"); ## no critic (ProhibitMagicNumbers)
    is($DatabaseBasketOrOrder->getUserID(), 99999, "$SubTestName - UserId has expected value"); ## no critic (ProhibitMagicNumbers)
    is($DatabaseBasketOrOrder->getUserPassword(), 0, "$SubTestName - UserPassword has expected value"); ## no critic (ProhibitMagicNumbers)
    is($DatabaseBasketOrOrder->getAdminActionPassword(), '5cfgRT34xsdedtFLdfHxj7tfwx24fe', "$SubTestName - AdminActionPassword has expected value");
    is($DatabaseBasketOrOrder->getCurrencyID(), 'EUR', "$SubTestName - CurrencyID has expected value");
    is($DatabaseBasketOrOrder->getGrandTotal(), 1.23, "$SubTestName - GrandTotal has expected value"); ## no critic (ProhibitMagicNumbers)
    is($DatabaseBasketOrOrder->getTransactionType(), 'auth', "$SubTestName - tests if transaction type is expected");
    is($DatabaseBasketOrOrder->getPaymentType(), 'cc', "$SubTestName - tests if Payment type is expected");
    is($DatabaseBasketOrOrder->getPaymentTypeName(), 'MyCreditCard', "$SubTestName - tests if Payment type name is MyCreditCard");
    is($DatabaseBasketOrOrder->getIsTestTransaction(), '1', "$SubTestName - tests if IsTestTransaction is expected");
    is($DatabaseBasketOrOrder->getGUID(), '53AC1BFE-5362-E914-FC55-AC1532072208', "$SubTestName - tests if GUID is expected");
    is($DatabaseBasketOrOrder->getBillingAddress()->alias(), 'BillingAddress', "$SubTestName - tests if BillingAddress is there");
    is($DatabaseBasketOrOrder->getPreAuthTimePeriodInSeconds(), '518400', "$SubTestName - tests if we get 6 Days in seconds - 518400");
    is($DatabaseBasketOrOrder->getTransTime(), 'DummyTransTime', "$SubTestName - tests if we get DummyTransTime");
    isa_ok($DatabaseBasketOrOrder->getLineItemPaymentIpayment(), 'DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment', "$SubTestName - test if the object is a LineItemPaymentIpayment");

    is($DatabaseBasketOrOrder->getHiddenTriggerUrl(), 'Called_BuildShopUrl_Mock', "$SubTestName - hidden trigger URL has expected value");
    is($DatabaseBasketOrOrder->getSilentErrorUrl(), 'Called_BuildShopUrl_Mock', "$SubTestName - hidden trigger URL has expected value");
    is($DatabaseBasketOrOrder->getRedirectUrl(), 'Called_BuildShopUrl_Mock', "$SubTestName - hidden trigger URL has expected value");
    is($DatabaseBasketOrOrder->getSelectedTransactionType(), 'preauth', "$SubTestName - Selected Transaction Type has expected value");

    is($DatabaseBasketOrOrder->getPartnerId(), 'testPartnerId', "$SubTestName - Selected Transaction Type has expected value");
    is($DatabaseBasketOrOrder->getEpagesVersion(), '6.6.6', "$SubTestName - Selected Epages Version has expected value");
    ## critic (ProhibitLongLines)
}



sub test_ExtractionWithInvalidBasketOrOrder {
    my $SubTestName = (caller(0))[3];

    my $MockedBasket = _GetInvalidBasket();
    my $DatabaseBasketOrOrder = _GetDatabaseBasketOrOrder();


    my $ExtractionWasSuccessful = $DatabaseBasketOrOrder->extractFromBasketOrOrder($MockedBasket);

    ok(!$ExtractionWasSuccessful, "$SubTestName - was it possible to extract data from the database object - no");

    throws_ok { $DatabaseBasketOrOrder->getAccountID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getUserID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getUserPassword() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getAdminActionPassword() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getCurrencyID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getGrandTotal() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getTransactionType() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getPaymentType() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getIsTestTransaction() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getPreAuthTimePeriodInSeconds() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getTransTime() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getSelectedTransactionType() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getGUID() } (
        qr/Can\'t call method \"guid\" on an undefined value at/,
        "$SubTestName - tests if Calling of the GUID-Getter is not possible"
        );

    ## no critic (ProhibitLongLines)
    is($DatabaseBasketOrOrder->getLineItemPaymentIpayment(), undef, "$SubTestName - tests if LineItemPaymentIpayment is undef");
    is($DatabaseBasketOrOrder->getHiddenTriggerUrl(), undef, "$SubTestName - hidden trigger URL has expected value");
    is($DatabaseBasketOrOrder->getSilentErrorUrl(), undef, "$SubTestName - Silent error URL has expected value");
    is($DatabaseBasketOrOrder->getRedirectUrl(), undef, "$SubTestName - redirect URL has expected value");
    is($DatabaseBasketOrOrder->getBillingAddress(), undef, "$SubTestName - BillingAddress has expected value");
    ## critic (ProhibitLongLines)

}



sub test_ExtractionWithNoBasket() {
    my $SubTestName = (caller(0))[3];

    my $MockedBasket = _GetNoBasket();
    my $DatabaseBasketOrOrder = _GetDatabaseBasketOrOrder();

    my $ExtractionWasSuccessful = $DatabaseBasketOrOrder->extractFromBasketOrOrder($MockedBasket);
    ok(!$ExtractionWasSuccessful, "$SubTestName - was it possible to extract data from the database object - no");

    throws_ok { $DatabaseBasketOrOrder->getAccountID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getUserID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getUserPassword() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getAdminActionPassword() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getCurrencyID() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getGrandTotal() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - Calling the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getTransactionType() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getPaymentType() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getIsTestTransaction() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getPreAuthTimePeriodInSeconds() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getTransTime() } (
        qr/Can\'t call method \"get\" on an undefined value at/,
        "$SubTestName - tests if Calling of the DatabaseBasketOrOrder-Getter is not possible"
        );

    throws_ok { $DatabaseBasketOrOrder->getGUID() } (
        qr/Can\'t call method \"guid\" on an undefined value at/,
        "$SubTestName - tests if Calling of the GUID-Getter is not possible"
        );

    ## no critic (ProhibitLongLines)
    is($DatabaseBasketOrOrder->getLineItemPaymentIpayment(), undef, "$SubTestName - tests if LineItemPaymentIpayment is undef");
    is($DatabaseBasketOrOrder->getHiddenTriggerUrl(), undef, "$SubTestName - hidden trigger URL has expected value");
    is($DatabaseBasketOrOrder->getSilentErrorUrl(), undef, "$SubTestName - Silent error URL has expected value");
    is($DatabaseBasketOrOrder->getRedirectUrl(), undef, "$SubTestName - redirect URL has expected value");
    is($DatabaseBasketOrOrder->getBillingAddress(), undef, "$SubTestName - BillingAddress has expected value");
    ## critic (ProhibitLongLines)

}

sub _GetMockedBasket {

    #PaymentTypeMethodsGroup
    my $hPaymentTypeMethodsGroupAttributes = {
    };
    my $MockedPaymentTypeMethodsGroup = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethodsGroup', $hPaymentTypeMethodsGroupAttributes
        );

    #PaymentTypeMethod
    my $hPaymentTypeMethodAttributes = {
        'Alias' => 'MyCreditCard',
        'TransactionTypes' => '{"TransactionTypes":["auth"]}',
        'ProductType' => 'cc',
        'Parent' => $MockedPaymentTypeMethodsGroup,
    };
    my $MockedPaymentTypeMethod = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethod', $hPaymentTypeMethodAttributes
        );

    #PaymentMethodIpayment
    my $hMockedPaymentMethodIpaymentAttributes = {
        'AccountID' => '99999',
        'UserID' => '99999',
        'UserPassword' => '0',
        'AdminActionPassword' => '5cfgRT34xsdedtFLdfHxj7tfwx24fe',
        'PaymentTypeMethod' => $MockedPaymentTypeMethod,
        'SelectedTransactionType' => 'auth',
        'PreAuthTimePeriodInDays' => '6',

    };
    my $MockedPaymentMethodIpayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::Object::PaymentMethodIpayment', $hMockedPaymentMethodIpaymentAttributes
        );

    #LineItemPaymentIpayment
    my $hMockedLineItemPaymentIpaymentAttributes = {
        'PaymentMethod' => $MockedPaymentMethodIpayment,
        'IsTestTransaction' => '1',
        'GUID' => '53AC1BFE-5362-E914-FC55-AC1532072208',
        'TransTime' => 'DummyTransTime',
        'SelectedTransactionType' => 'preauth',
    };
    my $MockedLineItemPaymentIpayment = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment', $hMockedLineItemPaymentIpaymentAttributes
        );
    $MockedLineItemPaymentIpayment->mock('guid', sub {
        my $self = shift;
        return $self->get('GUID');
    });

    #LineItemContainer 
    my $hMockedLineItemContainerAttributes = {
        'CurrencyID' => 'EUR',
        'GrandTotal' => '1.23',
        'Payment' => $MockedLineItemPaymentIpayment,
    };
    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer', $hMockedLineItemContainerAttributes
        );

    #BillingAddress 
    my $hMockedBillingAddressAttributes = {
        'Alias' => 'BillingAddress',
    };
    my $MockedBillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address', $hMockedBillingAddressAttributes
        );

    #Shop
    my $MockedShop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});

    #Basket
    my $hMockedBasketAttributes = {
        'LineItemContainer' => $MockedLineItemContainer,
        'Site' => $MockedShop,
        'BillingAddress' => $MockedBillingAddress
    };
    my $MockedBasket = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket', $hMockedBasketAttributes
        );

    return $MockedBasket;
}


sub _GetNoBasket {
    return undef;
}

sub _GetInvalidBasket {
    my $hMockedBasketAttributes = {
        'LineItemContainer' => '',
        'CurrencyID' => '',
        'GrandTotal' => '',
        'Site' => '',

    };
    my $InvalidMockedBasket = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket', $hMockedBasketAttributes
        );

    return $InvalidMockedBasket;
}

sub _GetDatabaseBasketOrOrder {
    my $DatabaseBasketOrOrderOriginal = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    my $DatabaseBasketOrOrder = Test::MockObject::Extends->new( $DatabaseBasketOrOrderOriginal );
    $DatabaseBasketOrOrder->mock('_BuildShopUrl',sub{
            my $self = shift;
            my ($Object, $hParams, $hOptions, $Servlet) = @_;
            return 'Called_BuildShopUrl_Mock';
    });
    $DatabaseBasketOrOrder->mock('_LoadRootObject', sub {
        my $hAttributes = {
            'Ipayment_PartnerId' => 'testPartnerId',
            'EpagesVersion' => '6.6.6',
        };
        my $MockedSystem = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site', $hAttributes);
        return $MockedSystem;
    });
    return $DatabaseBasketOrOrder;
}

RunScript( Sub => \&Main );